@@ -1,5 +1,5 @@
 package Git::Repository::Command;
-
+$Git::Repository::Command::VERSION = '1.311';
 use strict;
 use warnings;
 use 5.006;
@@ -7,16 +7,13 @@ use 5.006;
 use Carp;
 use Cwd qw( cwd );
 use IO::Handle;
-use IPC::Open3 qw( open3 );
 use Scalar::Util qw( blessed );
 use File::Spec;
 use Config;
 use System::Command;
 
-our $VERSION = '1.15';
 our @ISA = qw( System::Command );
 
-
 # a few simple accessors
 for my $attr (qw( pid stdin stdout stderr exit signal core )) {
     no strict 'refs';
@@ -64,7 +61,7 @@ sub _is_git {
         my @ext = (
             '', $^O eq 'MSWin32' ? ( split /\Q$path_sep\E/, $ENV{PATHEXT} ) : ()
         );
-        ($git) = grep { -e && !-d }
+        ($git) = grep { -x && !-d }
             map {
             my $path = $_;
             map { File::Spec->catfile( $path, $_ ) } map {"$binary$_"} @ext
@@ -80,9 +77,9 @@ sub _is_git {
         if !( defined $git && -x $git );
 
     # try to run it
-    my ( $pid, $in, $out, $err )
-        = System::Command->spawn( $git, @args, '--version' );
-    my $version = do { local $/ = "\n"; <$out>; };
+    my $cmd = System::Command->new( $git, @args, '--version' );
+    my $version = do { local $/ = "\n"; $cmd->stdout->getline; } || '';
+    $cmd->close;
 
     # does it really look like git?
     return $binary{$type}{$key}{$binary}{$args}
@@ -97,9 +94,15 @@ sub new {
     my ( $class, @cmd ) = @_;
 
     # split the args
-    my ($r) = grep { blessed $_ && $_->isa('Git::Repository') } @cmd;
-    my @o = grep { ref eq 'HASH' } @cmd;
-    @cmd = grep { !ref } @cmd;
+    my (@r, @o);
+    @cmd =    # take out the first Git::Repository in $r, and options in @o
+        grep !( blessed $_ && $_->isa('Git::Repository') ? push @r, $_ : 0 ),
+        grep !( ref eq 'HASH'                            ? push @o, $_ : 0 ),
+        @cmd;
+
+    # wouldn't know what to do with more than one Git::Repository object
+    croak "Too many Git::Repository objects given: @r" if @r > 1;
+    my $r = shift @r;
 
     # keep changes to the environment local
     local %ENV = %ENV;
@@ -122,6 +125,17 @@ sub new {
             if defined $work_tree;
     }
 
+    # extract and process the 'fatal' option
+    push @o, {
+        fatal => {
+            128 => 1,    # fatal
+            129 => 1,    # usage
+            map s/^-// ? ( $_ => '' ) : ( $_ => 1 ),
+            map /^!0$/ ? ( 1 .. 255 ) : $_,
+            map ref() ? @$_ : $_, grep defined, map $_->{fatal}, @o
+        }
+    };
+
     # get and check the git command
     my $git_cmd = ( map { exists $_->{git} ? $_->{git} : () } @o )[-1];
 
@@ -143,7 +157,7 @@ sub new {
 }
 
 sub final_output {
-    my ($self) = @_;
+    my ($self, @cb) = @_;
 
     # get output / errput
     my ( $stdout, $stderr ) = @{$self}{qw(stdout stderr)};
@@ -157,14 +171,18 @@ sub final_output {
     # done with it
     $self->close;
 
-    # exit codes: 128 => fatal, 129 => usage
-    my $exit = $self->{exit};
-    if ( $exit == 128 || $exit == 129 ) {
+    # fatal exit codes set by the 'fatal' option
+    if ( $self->options->{fatal}{ $self->exit } ) {
         croak join( "\n", @errput ) || 'fatal: unknown git error';
     }
 
     # something else's wrong
-    if (@errput) { carp join "\n", @errput; }
+    if ( @errput && !$self->options->{quiet} ) { carp join "\n", @errput; }
+
+    # process the output with the optional callbacks
+    for my $cb (@cb) {
+        @output = map $cb->($_), @output;
+    }
 
     # return the output
     return wantarray ? @output : join "\n", @output;
@@ -172,12 +190,20 @@ sub final_output {
 
 1;
 
+# ABSTRACT: Command objects for running git
+
+
 __END__
+=pod
 
 =head1 NAME
 
 Git::Repository::Command - Command objects for running git
 
+=head1 VERSION
+
+version 1.311
+
 =head1 SYNOPSIS
 
     use Git::Repository::Command;
@@ -208,26 +234,27 @@ Git::Repository::Command - Command objects for running git
     # cut to the chase
     my ( $pid, $in, $out, $err ) = Git::Repository::Command->spawn(@cmd);
 
-
 =head1 DESCRIPTION
 
-C<Git::Repository::Command> is a class that actually launches a B<git>
+L<Git::Repository::Command> is a class that actually launches a B<git>
 commands, allowing to interact with it through its C<STDIN>, C<STDOUT>
 and C<STDERR>.
 
-This class is a subclass of C<System::Command>, meant to be invoked
-through C<Git::Repository>.
+This class is a subclass of L<System::Command>, meant to be invoked
+through L<Git::Repository>.
 
 =head1 METHODS
 
-As a subclass of C<System::Command>,
-C<Git::Repository::Command> supports the following methods:
+As a subclass of L<System::Command>,
+L<Git::Repository::Command> supports the following methods:
 
-=head2 new( @cmd )
+=head2 new
+
+    Git::Repository::Command->new( @cmd );
 
 Runs a B<git> command with the parameters in C<@cmd>.
 
-If C<@cmd> contains a C<Git::Repository> object, it is used to provide
+If C<@cmd> contains a L<Git::Repository> object, it is used to provide
 context to the B<git> command.
 
 If C<@cmd> contains one or more hash reference, they are taken as
@@ -248,11 +275,32 @@ should be an array reference with the command and parameters, like this:
 =item C<cwd>
 
 The I<current working directory> in which the git command will be run.
+(C<chdir()> will be called just before lauching the command.)
+
+If not provided, it will default to the root of the Git repository work
+tree (if the repository is bare, then no C<chdir()> will be performed).
 
 =item C<env>
 
 A hashref containing key / values to add to the git command environment.
 
+=item C<fatal>
+
+An arrayref containing a list of exit codes that will be considered
+fatal by C<final_output()>.
+
+Prepending the value with C<-> will make it non-fatal, which can be
+useful to override a default. The string C<"!0"> can be used as a
+shortcut for C<[ 1 .. 255 ]>.
+
+If several option hashes have the C<fatal> key, the lists of exit codes
+will be combined, with the values provided last taking precedence (when
+using a combination of positive / negative values).
+
+The generated list always contains C<128> and C<129>; to make them
+non-fatal, just add C<-128> and C<-129> to the list provided to the
+C<fatal> option.
+
 =item C<input>
 
 A string that is send to the git command standard input, which is then closed.
@@ -265,28 +313,38 @@ a way to modify options inherited from C<new()> or a hash populated by
 some other part of the program.
 
 On some systems, some git commands may close standard input on startup,
-which will cause a SIGPIPE when trying to write to it. This will raise
+which will cause a C<SIGPIPE> when trying to write to it. This will raise
 an exception.
 
+=item C<quiet>
+
+Boolean option to control the output of warnings.
+
+If true, methods such as C<final_output()> will not warn when Git outputs
+messages on C<STDERR>.
+
 =back
 
-If the C<Git::Repository> object has its own option hash, it will be used
+If the L<Git::Repository> object has its own option hash, it will be used
 to provide default values that can be overridden by the actual option hash
 passed to C<new()>.
 
 If several option hashes are passed to C<new()>, they will all be merged,
 keys in later hashes taking precedence over keys in earlier hashes.
 
-The C<Git::Repository::Command> object returned by C<new()> has a
+The L<Git::Repository::Command> object returned by C<new()> has a
 number of attributes defined (see below).
 
+=head2 close
 
-=head2 close()
+    $cmd->close();
 
 Close all pipes to the child process, and collects exit status, etc.
 and defines a number of attributes (see below).
 
-=head2 final_output()
+=head2 final_output
+
+    $cmd->final_output( @callbacks );
 
 Collect all the output, and terminate the command.
 
@@ -295,37 +353,41 @@ or as a list of lines in list context. Also accepts a hashref of options.
 
 Lines are automatically C<chomp>ed.
 
-If the Git command printed anything on stderr, it will be printed as
-warnings. If the git sub-process exited with status C<128> (fatal error),
-or C<129> (usage message), it will C<die()>.
+If C<@callbacks> is provided, the code references will be applied
+successively to each line of output. The line being processed is in C<$_>,
+but the coderef must still return the result string.
 
+If the Git command printed anything on stderr, it will be printed as
+warnings. If the git sub-process exited with a status code listed in
+the C<fatal> option, it will C<die()>. The defaults fatal exit codes
+are C<128> (fatal error), and C<129> (usage message).
 
 =head2 Accessors
 
-The attributes of a C<Git::Repository::Command> object are also accessible
+The attributes of a L<Git::Repository::Command> object are also accessible
 through a number of accessors.
 
 The object returned by C<new()> will have the following attributes defined:
 
 =over 4
 
-=item cmdline()
+=item cmdline
 
 Return the command-line actually executed, as a list of strings.
 
-=item pid()
+=item pid
 
 The PID of the underlying B<git> command.
 
-=item stdin()
+=item stdin
 
 A filehandle opened in write mode to the child process' standard input.
 
-=item stdout()
+=item stdout
 
 A filehandle opened in read mode to the child process' standard output.
 
-=item stderr()
+=item stderr
 
 A filehandle opened in read mode to the child process' standard error output.
 
@@ -337,36 +399,30 @@ following code:
     my $fh = Git::Repository::Command->new( @cmd )->stdout;
 
 C<$fh> is opened and points to the output of the git subcommand, while
-the anonymous C<Git::Repository::Command> object has been destroyed.
-Once C<$fh> is destroyed, the subprocess will be reaped, thus avoiding
-zombies.
+the anonymous L<Git::Repository::Command> object has been destroyed.
 
 After the call to C<close()>, the following attributes will be defined:
 
 =over 4
 
-=item exit()
+=item exit
 
 The exit status of the underlying B<git> command.
 
-=item core()
+=item core
 
 A boolean value indicating if the command dumped core.
 
-=item signal()
+=item signal
 
 The signal, if any, that killed the command.
 
 =back
 
-=head1 AUTHOR
-
-Philippe Bruhat (BooK), C<< <book at cpan.org> >>
-
 =head1 ACKNOWLEDGEMENTS
 
-The core of C<Git::Repository::Command> has been moved into its own
-distribution: C<System::Command>. Proper Win32 support is now delegated
+The core of L<Git::Repository::Command> has been moved into its own
+distribution: L<System::Command>. Proper Win32 support is now delegated
 to that module.
 
 Before that, the Win32 implementation owed a lot to two people.
@@ -376,9 +432,28 @@ Many thanks go also to Chris Williams (BINGOS) for pointing me towards
 perlmonks posts by ikegami that contained crucial elements to a working
 MSWin32 implementation.
 
+In the end, it was Christian Walder (MITHALDU) who helped me finalize
+Win32 support for L<System::Command> through a quick round of edit
+(on my Linux box) and testing (on his Windows box) during the Perl QA
+Hackathon 2013 in Lancaster.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+http://rt.cpan.org/NoAuth/Bugs.html?Dist=Git-Repository or by email to
+bug-git-repository@rt.cpan.org.
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
+=head1 AUTHOR
+
+Philippe Bruhat (BooK) <book@cpan.org>
+
 =head1 COPYRIGHT
 
-Copyright 2010-2011 Philippe Bruhat (BooK), all rights reserved.
+Copyright 2010-2014 Philippe Bruhat (BooK), all rights reserved.
 
 =head1 LICENSE
 
